<?php

class WC_Gateway_FSWW extends WC_Payment_Gateway {
    
    public function __construct() {
        
        $this->id                       = 'fsww';
        $this->icon                     = apply_filters('woocommerce_cod_icon', '');
        $this->method_title             = __('WooCommerce Wallet', 'fsww');
        $this->methode_description      = __('Pay with your account balance.', 'fsww');
        $this->has_fields               = false;
        
        $this->init_form_fields();
        $this->init_settings();
        
        $this->title                    = $this->get_option('title');
        $this->description              = $this->format_description();
        $this->instuctions              = $this->get_option('instuctions');
        $this->enable_for_methods       = $this->get_option('enable_for_methods', array());
        $this->enable_for_virtual       = $this->get_option('enable_for_virtual', 'yes') == 'yes' ? true : false;
        
        // registe actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'));
        add_action('woocommerce_before_checkout_process' . $this->id, array($this, 'check_balance'));
        
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
        
    }
    
    public function format_description() {
        
        global $woocommerce;
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        
        $balance = wc_price(Wallet::get_balance(get_current_user_id()));

        $description  = '<p>' . __('Available Balance:', 'fsww') . ' ' .$balance . '</p>';
	
        
        $description .= $this->get_option('description');
        
        return $description;
        
    }
    
    public function init_form_fields() {
        $shipping_methods = array();
        
        if(is_admin()) {
            foreach(WC()->shipping->load_shipping_methods() as $method) {
                $shipping_methods[$method->id] = $method->get_title();
            }
        }
        
        $this->form_fields = array(
            'enabled' => array(
                'title'       => __('Enable WooCommerce Wallet', 'woocommerce'),
                'lable'       => __('Enable WooCommerce Wallet', 'woocommerce'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'title' => array(
				'title'       => __('Title', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Payment method description that the customer will see on your checkout.', 'woocommerce'),
				'default'     => __('WooCommerce Wallet', 'woocommerce'),
				'desc_tip'    => true,
			),
			'description' => array(
				'title'       => __('Description', 'woocommerce'),
				'type'        => 'textarea',
				'description' => __('Payment method description that the customer will see on your website.', 'woocommerce'),
				'default'     => __('Pay using your Wallet.', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'instructions' => array(
				'title'       => __('Instructions', 'woocommerce'),
				'type'        => 'textarea',
				'description' => __('Instructions that will be added to the thank you page.', 'woocommerce'),
				'default'     => __('Pay using your Wallet', 'woocommerce'),
				'desc_tip'    => true,
			),
			'enable_for_methods' => array(
				'title'             => __('Enable for shipping methods', 'woocommerce'),
				'type'              => 'multiselect',
				'class'             => 'chosen_select',
				'css'               => 'width: 450px;',
				'default'           => '',
				'description'       => __('If WooCommerce Wallet is only available for certain shipping methods, set it up here. Leave blank to enable for all methods.', 'woocommerce'),
				'options'           => $shipping_methods,
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __('Select shipping methods', 'woocommerce')
				)
			),
			'enable_for_virtual' => array(
				'title'             => __('Enable for virtual orders', 'woocommerce'),
				'label'             => __('Enable WooCommerce Wallet for virtual products', 'woocommerce'),
				'type'              => 'checkbox',
				'default'           => 'yes'
			)
        );        
        
    }
    
    public function process_payment($order_id) {
        
        global $woocommerce;
        
        foreach($woocommerce->cart->get_cart() as $cart_item_key => $cart_item){
            
            if(has_term('fsww-wallet-credit', 'product_cat', $cart_item['product_id'])){
                
                wc_add_notice(__('<strong>Payment error:</strong> You can not purchase wallet credit money with wallet credit. Please choose another payment method.', 'fsww') , 'error' );
             
                return false;
                
            }
            
        }
        
        if(!$this->check_balance()) {
            
            wc_add_notice( __('<strong>Payment error:</strong> Insufficient funds. Please purchase more credits or use a different payment method.', 'fsww'), 'error' );
            
            return false;
            
        }
        
        $order         = wc_get_order($order_id);
        $user_id       = $order->get_user_id();
        
        
        $balance       = floatval(Wallet::get_balance(get_current_user_id()));
        $balance_new   = $balance - floatval($woocommerce->cart->total);
        
        Wallet::withdraw_funds($user_id, $woocommerce->cart->total, $order_id);
        Wallet::add_spending($user_id, floatval($woocommerce->cart->total));
        
        
        $balance       = floatval(Wallet::get_balance(get_current_user_id()));
        
        if($balance != $balance_new) {
            
            wc_add_notice(__('<strong>System error:</strong> There was an error procesing the payment. Please try another payment method.', 'fsww'), 'error');
			
            return false;
            
        }
        
        $order->update_status(get_option('fsww_order_status', 'completed'), __('Order status set to ' . get_option('fsww_order_status', 'completed') . ' by WooCommerce Wallet', 'fsww'));
        
        
        $order->reduce_order_stock();
        
        WC()->cart->empty_cart();
		
        
        return array(
			'result' 	=> 'success',
			'redirect'	=> $this->get_return_url($order)
		);
        
    }
    
    public function check_balance() {
        
        global $woocommerce;
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $balance  = floatval(Wallet::get_balance(get_current_user_id()));
        $total    = floatval($woocommerce->cart->total);
        
        if($balance >= $total) {
                
            return 1;
            
        }
        
        return 0;
        
    }
    
    public function is_available() {
		
        $order = null;
        
        
        if(!is_user_logged_in()) {
            
            return false;
        
        }
        
        if($this->locked_account()) {
          
            return false;
            
        }
        
        $fees     = WC()->cart->get_fees();
        
        foreach($fees as $fee) {
            
            if($fee->tax_class == 'fsww_pfw') {
             
                return false;
                
            }
            
        }       
        
        
        global $woocommerce;
        
        foreach($woocommerce->cart->get_cart() as $cart_item_key => $cart_item){
            
            if(has_term('fsww-wallet-credit', 'product_cat', $cart_item['product_id'])){
                
                return false;
                
            }
            
        }
        
        if(!$this->enable_for_virtual) {

            if(WC()->cart && WC()->cart->needs_shipping()) {
                
                return false;
                
            }
            
            if(is_page(wc_get_page_id('checkout')) && get_query_var('order-pay') > 0) {
                
                $order_id   = absint(get_query_var('order-pay'));                    
                $order      = wc_get_order($order_id);
                
                $needs_shipping = false;
				if (0 < sizeof($order->get_items())) {
                    
					foreach ($order->get_items() as $item) {
                        
						$_product = $order->get_product_from_item($item);

						if ( $_product->needs_shipping()) {
                            
							$needs_shipping = true;
							break;
                            
						}
                        
					}
                    
				}
                
				$needs_shipping = apply_filters('woocommerce_cart_needs_shipping', $needs_shipping);
				if ($needs_shipping) {
                    
                    return false;
                    
                }
                
            }
            
        }
        
        
        if(!empty($this->enable_for_methods)) {
            
            $chosen_shipping_methods = array();
            $chosen_shipping_methods_session = WC()->session->get('chosen_shipping_methods');
            
            if(isset($chosen_shipping_methods_session))	{
                
				$chosen_shipping_methods = array_unique($chosen_shipping_methods_session);
                
			}

			$check_method = false;

			if(is_object($order)) {
                
				if($order->shipping_method) {
					
                    $check_method = $order->shipping_method;
                    
				}

			} elseif (empty($chosen_shipping_methods) || sizeof($chosen_shipping_methods) > 1) {
                
				$check_method = false;
                
			} elseif (sizeof($chosen_shipping_methods) == 1) {
                
				$check_method = $chosen_shipping_methods[0];
                
			}

			if (!$check_method) {
                
                return false;
                
            }
				
			$found = false;

			foreach($this->enable_for_methods as $method_id ) {
				
                if(strpos($check_method, $method_id) === 0) {
                    
					$found = true;
					break;
                    
				}
                
			}

			if (!$found) {
                return false;
            }
				
        }
        
        
        return parent::is_available();
        
    }
    
    public function locked_account() {
        
        require_once(dirname(__FILE__) . '/Wallet.php');
        
        $status   = Wallet::lock_status(get_current_user_id());
        
        if($status['status'] == 'locked') {
                
            return true;
            
        }
        
        return false;
        
    }
    
    public function get_icon() {
        
        $icon = '';

		return apply_filters('woocommerce_gateway_icon', $icon, $this->id);
	
    }
    
    public function thankyou_page() {
        
        if(isset($this->instructions)) {
            wpautop(wptexturize($this->instructions));
        }
        
    }
    
    public function email_instructions($order, $send_to_admin, $plain_text = false) {
        
        if($this->instructions && $send_to_admin && 'fsww' == $order->payment_method) {
            echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
        }
        
    }
    
}